#include "capy.h"
#ifndef FIXTURE
#define FIXTURE
#endif
CUTEST(test001, "Alloc/free") {
  CapyDate* date = CapyDateAlloc();
  CUTEST_ASSERT(
    date->vals[capyDateUnit_year] == 0 &&
    date->vals[capyDateUnit_month] == 0 &&
    date->vals[capyDateUnit_day] == 0 &&
    date->vals[capyDateUnit_hour] == 0 &&
    date->vals[capyDateUnit_minute] == 0 &&
    date->vals[capyDateUnit_second] == 0,
    "CapyDateAlloc failed");
  CapyDateFree(&date);
}

CUTEST(test002, "Set from and get as uint64_t") {
  CapyDate* date = CapyDateAlloc();
  $(date, setFromUInt64)(19770531193000UL);
  uint64_t d = $(date, getAsUInt64)();
  CUTEST_ASSERT(
    d == 19770531193000UL &&
    date->vals[capyDateUnit_year] == 1977 &&
    date->vals[capyDateUnit_month] == 4 &&
    date->vals[capyDateUnit_day] == 30 &&
    date->vals[capyDateUnit_hour] == 19 &&
    date->vals[capyDateUnit_minute] == 30 &&
    date->vals[capyDateUnit_second] == 0,
    "setFromUInt64 failed %lu (%d,%d,%d,%d,%d,%d)",
    d,
    date->vals[capyDateUnit_year],
    date->vals[capyDateUnit_month],
    date->vals[capyDateUnit_day],
    date->vals[capyDateUnit_hour],
    date->vals[capyDateUnit_minute],
    date->vals[capyDateUnit_second]);
  CapyDateFree(&date);
}

CUTEST(test003, "Get delay") {
  CapyDate* date = CapyDateAlloc();
  $(date, setFromUInt64)(19770531193000UL);
  CapyDate* dateTo = CapyDateAlloc();
  $(dateTo, setFromUInt64)(20240301111900UL);
  int64_t delay[capyDateUnit_nb] = {0};
  loop(i, capyDateUnit_nb) delay[i] = $(date, getDelayTo)(dateTo, i);
  CUTEST_ASSERT(
    delay[capyDateUnit_year] == 46 &&
    delay[capyDateUnit_month] == 560 &&
    delay[capyDateUnit_day] == 16829 &&
    delay[capyDateUnit_hour] == 403911 &&
    delay[capyDateUnit_minute] == 24234709 &&
    delay[capyDateUnit_second] == 1454082540,
    "getDelayTo failed (%ld,%ld,%ld,%ld,%ld,%ld)",
    delay[capyDateUnit_year],
    delay[capyDateUnit_month],
    delay[capyDateUnit_day],
    delay[capyDateUnit_hour],
    delay[capyDateUnit_minute],
    delay[capyDateUnit_second]);
  CapyDateFree(&date);
  CapyDateFree(&dateTo);
}

CUTEST(test004, "Get week day") {
  CapyDate* date = CapyDateAlloc();
  $(date, setFromUInt64)(19770531000000UL);
  CapyWeekDay day = $(date, getWeekDay)();
  CUTEST_ASSERT(
    day == capyWeekDay_tuesday, "getWeekDay failed %d", day);
  CapyDateFree(&date);
}

CUTEST(test005, "Add date (1)") {
  CapyDate delay = {.vals = {10, 9, 8, 7, 6, 5}};
  CapyDate* date = CapyDateAlloc();
  $(date, setFromUInt64)(19770531193000UL);
  $(date, addDate)(&delay);
  CUTEST_ASSERT(
    date->vals[capyDateUnit_year] == 1988 &&
    date->vals[capyDateUnit_month] == 2 &&
    date->vals[capyDateUnit_day] == 9 &&
    date->vals[capyDateUnit_hour] == 2 &&
    date->vals[capyDateUnit_minute] == 36 &&
    date->vals[capyDateUnit_second] == 5,
    "addDate failed (%d,%d,%d,%d,%d,%d)",
    date->vals[capyDateUnit_year],
    date->vals[capyDateUnit_month],
    date->vals[capyDateUnit_day],
    date->vals[capyDateUnit_hour],
    date->vals[capyDateUnit_minute],
    date->vals[capyDateUnit_second]);
  CapyDateFree(&date);
}

CUTEST(test006, "Add date (2)") {
  CapyDate delay = {.vals = {-10, -9, -8, -7, -6, -5}};
  CapyDate* date = CapyDateAlloc();
  $(date, setFromUInt64)(19770531193000UL);
  $(date, addDate)(&delay);
  CUTEST_ASSERT(
    date->vals[capyDateUnit_year] == 1966 &&
    date->vals[capyDateUnit_month] == 7 &&
    date->vals[capyDateUnit_day] == 22 &&
    date->vals[capyDateUnit_hour] == 12 &&
    date->vals[capyDateUnit_minute] == 23 &&
    date->vals[capyDateUnit_second] == 55,
    "addDate failed (%d,%d,%d,%d,%d,%d)",
    date->vals[capyDateUnit_year],
    date->vals[capyDateUnit_month],
    date->vals[capyDateUnit_day],
    date->vals[capyDateUnit_hour],
    date->vals[capyDateUnit_minute],
    date->vals[capyDateUnit_second]);
  CapyDateFree(&date);
}
