#include "capy.h"
#ifndef FIXTURE
#define FIXTURE
#endif
CUTEST(test001, "Triangle") {
  CapyTriangle* tri = CapyTriangleAlloc();
  CUTEST_ASSERT(equal(tri->corners[0].x, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(tri->corners[0].y, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(tri->corners[1].x, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(tri->corners[1].y, 1.0), "initialisation failed");
  CUTEST_ASSERT(equal(tri->corners[2].x, 1.0), "initialisation failed");
  CUTEST_ASSERT(equal(tri->corners[2].y, 0.0), "initialisation failed");
  double area = $(tri, getArea)();
  CUTEST_ASSERT(equal(area, 0.5), "getArea failed %lf!=0.5", area);
  CapyTriangleFree(&tri);
  CUTEST_ASSERT(tri == NULL, "tri not reset");
}

CUTEST(test002, "Quadrilateral") {
  CapyQuadrilateral* quad = CapyQuadrilateralAlloc();
  CUTEST_ASSERT(equal(quad->corners[0].x, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(quad->corners[0].y, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(quad->corners[1].x, 1.0), "initialisation failed");
  CUTEST_ASSERT(equal(quad->corners[1].y, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(quad->corners[2].x, 1.0), "initialisation failed");
  CUTEST_ASSERT(equal(quad->corners[2].y, 1.0), "initialisation failed");
  CUTEST_ASSERT(equal(quad->corners[3].x, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(quad->corners[3].y, 1.0), "initialisation failed");
  double area = $(quad, getArea)();
  CUTEST_ASSERT(equal(area, 1.0), "getArea failed %lf!=1.0", area);
  CapyQuadrilateralFree(&quad);
  CUTEST_ASSERT(quad == NULL, "quad not reset");
}

CUTEST(test003, "QuadrilateralBilinearCoord") {
  CapyQuadrilateral quad = CapyQuadrilateralCreate();
  double pos[2] = {0.75, 1.25};
  double coords[2] = {0.0, 0.0};
  $(&quad, getBilinearCoords)(pos, coords);
  CUTEST_ASSERT(
    equal(coords[0], pos[0]), "getBilinearCoords failed %lf", coords[0]);
  CUTEST_ASSERT(
    equal(coords[1], pos[1]), "getBilinearCoords failed %lf", coords[1]);
  $(&quad, destruct)();
}

CUTEST(test004, "Padding (1)") {
  CUTEST_ASSERT(
    offsetof(CapyPoint2D, x) == offsetof(CapyPoint2D, coords[0]),
    "offset of x and coords[0] doesn't match %lu!=%lu",
    offsetof(CapyPoint2D, x), offsetof(CapyPoint2D, coords[0]));
  CUTEST_ASSERT(
    offsetof(CapyPoint2D, y) == offsetof(CapyPoint2D, coords[1]),
    "offset of y and coords[1] doesn't match %lu!=%lu",
    offsetof(CapyPoint2D, y), offsetof(CapyPoint2D, coords[1]));
}

CUTEST(test005, "Padding (2)") {
  CUTEST_ASSERT(
    offsetof(CapyPoint2D, x) ==
      offsetof(CapyPoint2D, coords[0]),
    "offset of x and coords[0] doesn't match %lu!=%lu",
    offsetof(CapyPoint2D, x),
    offsetof(CapyPoint2D, coords[0]));
  CUTEST_ASSERT(
    offsetof(CapyPoint2D, y) ==
      offsetof(CapyPoint2D, coords[1]),
    "offset of y and coords[1] doesn't match %lu!=%lu",
    offsetof(CapyPoint2D, y),
    offsetof(CapyPoint2D, coords[1]));
}

CUTEST(test006, "Circle") {
  CapyCircle* circle = CapyCircleAlloc();
  CUTEST_ASSERT(equal(circle->center.x, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(circle->center.y, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(circle->radius, 1.0), "initialisation failed");
  double area = $(circle, getArea)();
  CUTEST_ASSERT(equal(area, M_PI), "getArea failed %lf!=pi", area);
  CapyCircleFree(&circle);
  CUTEST_ASSERT(circle == NULL, "circle not reset");
}

CUTEST(test007, "Segment2D") {
  CapySegment* seg = CapySegmentAlloc();
  CUTEST_ASSERT(equal(seg->points[0].x, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(seg->points[0].y, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(seg->points[1].x, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(seg->points[1].y, 1.0), "initialisation failed");
  double area = $(seg, getArea)();
  CUTEST_ASSERT(equal(area, 0.0), "getArea failed %lf!=0.0", area);
  CapySegmentFree(&seg);
  CUTEST_ASSERT(seg == NULL, "seg not reset");
}

CUTEST(test008, "Rectangle") {
  CapyRectangle* rect = CapyRectangleAlloc();
  CUTEST_ASSERT(equal(rect->corners[0].x, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(rect->corners[0].y, 0.0), "initialisation failed");
  CUTEST_ASSERT(equal(rect->corners[1].x, 1.0), "initialisation failed");
  CUTEST_ASSERT(equal(rect->corners[1].y, 1.0), "initialisation failed");
  double area = $(rect, getArea)();
  CUTEST_ASSERT(equal(area, 1.0), "getArea failed %lf!=1.0", area);
  CapyRectangleFree(&rect);
  CUTEST_ASSERT(rect == NULL, "rect not reset");
}

CUTEST(test009, "Triangle barycentric coordinates") {
  CapyTriangle* tri = CapyTriangleAlloc();
  tri->corners[0].x = 0.0;
  tri->corners[0].y = 0.0;
  tri->corners[1].x = 1.0;
  tri->corners[1].y = 0.0;
  tri->corners[2].x = 0.0;
  tri->corners[2].y = 1.0;
  double coords[5][3] = {0};
  CapyPoint2D pos[5] = {
    {.x = 0.0, .y = 0.0},
    {.x = 1.0, .y = 0.0},
    {.x = 0.0, .y = 1.0},
    {.x = 0.5, .y = 0.0},
    {.x = 0.0, .y = 0.5},
  };
  loop(i, 5) $(tri, getBarycentricCoord)(pos + i, coords[i]);
  double check[5][3] = {
    {1, 0, 0}, {0, 1, 0}, {0, 0, 1}, {0.5, 0.5, 0}, {0.5, 0, 0.5}
  };
  bool ok = true;
  loop(i, 5) {
    ok &= (
      equal(coords[i][0], check[i][0]) &&
      equal(coords[i][1], check[i][1]) &&
      equal(coords[i][2], check[i][2]));
  }
  CUTEST_ASSERT(ok, "getBarycentricCoord failed");
  CapyTriangleFree(&tri);
}

