#include "capy.h"
#ifndef FIXTURE
#define FIXTURE
static void FunDivergence(
  double const* const in,
        double* const out) {
  out[0] = 4.0 * in[1] / (in[0] * in[0]);
  out[1] = sin(in[1]);
  out[2] = 3.0;
}

static void FunIntegral(
  double const* const in,
        double* const out) {
  out[0] = in[0] * in[1];
  out[1] = 3.0;
}

#endif
CUTEST(test001, "Padding") {
  CUTEST_ASSERT(
    offsetof(CapyMathFun, dimIn) == offsetof(CapyMathFun, dims[0]),
    "offset of dimIn and dims[0] doesn't match %lu!=%lu",
    offsetof(CapyMathFun, dimIn), offsetof(CapyMathFun, dims[0]));
  CUTEST_ASSERT(
    offsetof(CapyMathFun, dimOut) == offsetof(CapyMathFun, dims[1]),
    "offset of dimOut and dims[1] doesn't match %lu!=%lu",
    offsetof(CapyMathFun, dimOut), offsetof(CapyMathFun, dims[1]));
}

CUTEST(test002, "Divergence") {
  CapyMathFun fun = CapyMathFunCreate(3, 3);
  fun.eval = FunDivergence;
  double in[3] = {1, 0, 0};
  double div = $(&fun, evalDivergence)(in);
  double checkDiv = -8.0 * in[1] * pow(in[0], -3.0) + cos(in[1]);
  CUTEST_ASSERT(
    fabs(div - checkDiv) < 1e-6, "div=%lf != %lf", div, checkDiv);
  $(&fun, destruct)();
}

CUTEST(test003, "EvalIntegral") {
  CapyMathFun fun = CapyMathFunCreate(2, 2);
  fun.eval = FunIntegral;
  double out[2] = {0, 0};
  CapyRangeDouble domains[2] = {
    CapyRangeDoubleCreate(0.0, 1.0),
    CapyRangeDoubleCreate(1.0, 2.0),
  };
  $(&fun, evalIntegral)(domains, out);
  double check[2] = {0.75, 3.0};
  loop(i, 2) {
    CUTEST_ASSERT(
      fabs(out[i] - check[i]) < 1e-6, "integral=%lf != %lf", out[i], check[i]);
  }
  $(&fun, destruct)();
}
