#include "capy.h"
#ifndef FIXTURE
#define FIXTURE
static void OscillatorDerivative(
  double const* const in,
        double* const out) {
  double const k = 1.5;
  double const m = 0.5;
  out[0] = -k / m * in[0] * in[1];
}

static double Oscillator(double const t) {
  double const k = 1.5;
  double const m = 0.5;
  double const a = 1.0;
  double const phi = M_PI_2;
  return a * sin(sqrt(k / m) * t + phi);
}

#endif
CUTEST(test001, "Harmonic oscillator, eval") {
  CapyMathFun derivative = CapyMathFunCreate(2, 1);
  derivative.eval = OscillatorDerivative;
  CapyRungeKutta* rk = CapyRungeKuttaAlloc(&derivative);
  rk->initVals[0] = 1.0;
  double const t = rk->deltaT * 100.0;
  double x = $(rk, eval)(t);
  double check = Oscillator(t);
  CUTEST_ASSERT(
    fabs(x - check) < 1e-4,
    "%lf!=%lf\n", x, check);
  CapyRungeKuttaFree(&rk);
  $(&derivative, destruct)();
}

CUTEST(test002, "Harmonic oscillator, evalRange") {
  CapyMathFun derivative = CapyMathFunCreate(2, 1);
  derivative.eval = OscillatorDerivative;
  CapyRungeKutta* rk = CapyRungeKuttaAlloc(&derivative);
  rk->initVals[0] = 1.0;
  double const t = rk->deltaT * 100.0;
  CapyVec res = $(rk, evalRange)(t);
  bool isOk = (res.dim == 101);
  if(isOk) loop(iEval, res.dim) {
    double const check =
      Oscillator(rk->initVals[1] + ((double)iEval) * rk->deltaT);
    isOk &= (fabs(res.vals[iEval] - check) < 1e-4);
  }
  CUTEST_ASSERT(isOk, "isOk=%d\n", isOk);
  CapyRungeKuttaFree(&rk);
  $(&derivative, destruct)();
  CapyVecDestruct(&res);
}
