#include "capy.h"
#ifndef FIXTURE
#define FIXTURE
#endif
CUTEST(test001, "Alloc and free") {
  CapyTurtleGraphic* turtle = CapyTurtleGraphicAlloc();
  CUTEST_ASSERT(
    fabs(turtle->state.step.vals[0] - 1.0) < 1e-6 &&
    fabs(turtle->state.step.vals[1] - 0.0) < 1e-6 &&
    fabs(turtle->state.pos.vals[0] - 0.0) < 1e-6 &&
    fabs(turtle->state.pos.vals[1] - 0.0) < 1e-6,
    "turtle creation failed");
  CapyTurtleGraphicFree(&turtle);
}

CUTEST(test002, "Koch snowflake") {
  CapyImg* img = CapyImgAlloc(capyImgMode_rgb, capyImgDims_800x600);
  CapyLSysKochSnowflakeState init = {.val = 'F'};
  CapyLSysKochSnowflake* koch = CapyLSysKochSnowflakeAlloc(&init, 1);
  loop(i, 6) $(koch, step)();
  CapyTurtleGraphic* turtle = CapyTurtleGraphicAlloc();
  turtle->img = img;
  turtle->state.pos.vals[1] = 100;
  $(turtle, resize)(800.0);
  loop(i, koch->len) {
    switch(koch->state[i].val) {
      case 'D':
        $(turtle, resize)(1.0 / 3.0);
        break;
      case 'U':
        $(turtle, resize)(3.0);
        break;
      case 'F':
        $(turtle, forward)(1);
        break;
      case '+':
        $(turtle, turn)(M_PI / 3.0);
        break;
      case '-':
        $(turtle, turn)(-M_PI / 3.0);
        break;
      default:
        break;
    }
  }
  $(img, saveToPath)("UnitTests/TestTurtleGraphic/koch.png");
  CapyImg* checkImg =
    CapyImgLoadFromPath("UnitTests/TestTurtleGraphic/checkKoch.png");
  CUTEST_ASSERT($(img, isSame)(checkImg), "turtle on Koch snowflake failed");
  CapyTurtleGraphicFree(&turtle);
  CapyLSysKochSnowflakeFree(&koch);
  CapyImgFree(&img);
  CapyImgFree(&checkImg);
}

CUTEST(test003, "Fractal binary tree") {
  CapyImg* img = CapyImgAlloc(capyImgMode_rgb, capyImgDims_800x600);
  CapyLSysFractalBinaryTreeState init = {.val = '0'};
  CapyLSysFractalBinaryTree* tree = CapyLSysFractalBinaryTreeAlloc(&init, 1);
  loop(i, 6) $(tree, step)();
  CapyTurtleGraphic* turtle = CapyTurtleGraphicAlloc();
  turtle->img = img;
  turtle->state.pos.vals[0] = 400;
  turtle->state.pos.vals[1] = 600;
  turtle->state.step.vals[0] = 0;
  turtle->state.step.vals[1] = -10;
  loop(i, tree->len) {
    switch(tree->state[i].val) {
      case '0':
        $(turtle, forward)(1);
        break;
      case '1':
        $(turtle, forward)(1);
        break;
      case '[':
        $(turtle, push)();
        $(turtle, turn)(M_PI / 4.0);
        break;
      case ']':
        $(turtle, pop)();
        $(turtle, turn)(-M_PI / 4.0);
        break;
      default:
        break;
    }
  }
  $(img, saveToPath)("UnitTests/TestTurtleGraphic/tree.png");
  CapyImg* checkImg =
    CapyImgLoadFromPath("UnitTests/TestTurtleGraphic/checkTree.png");
  CUTEST_ASSERT(
    $(img, isSame)(checkImg), "turtle on fractal binary tree failed");
  CapyTurtleGraphicFree(&turtle);
  CapyLSysFractalBinaryTreeFree(&tree);
  CapyImgFree(&img);
  CapyImgFree(&checkImg);
}
